==> 3.0.0.tar.gz: lib/ui/views/signinForm.dart <==
import 'package:account_kit/entities/request/request.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/user/userActions.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:puma_ui/ui/widgets/apiButton.dart';
import 'package:redux/redux.dart';

class SigninForm<T extends VirtualAccountState> extends StatefulWidget {
  // final void Function(String, String) onSubmit;
  final bool plEnabled;
  SigninForm({this.plEnabled = false});

  @override
  _SigninFormStateWidget createState() => _SigninFormStateWidget<T>();
}

/// This is the private State class that goes with SigninForm.
class _SigninFormStateWidget<T extends VirtualAccountState>
    extends State<SigninForm> {
  final _formKey = GlobalKey<FormState>();
  TextEditingController emailController = TextEditingController();
  TextEditingController codeController = TextEditingController();
  TextEditingController passwordController = TextEditingController();
  final FocusNode _emailFocus = FocusNode();
  final FocusNode _codeFocus = FocusNode();
  final FocusNode _passwordFocus = FocusNode();
  String _email = '';
  String _code = '';
  bool preventAutofocus = false;
  bool plEnabled = false;
  // Todo: Get Env default PLEnabled

  @override
  void initState() {
    super.initState();
    plEnabled = widget.plEnabled;
  }

  handleEmailChange(value) {
    setState(() {
      _email = value;
    });
  }

  handleCodeChange(value) {
    setState(() {
      _code = value;
    });
  }

  togglePlEnabled(_ViewModel viewModel) {
    if (plEnabled) {
      print("Dispatching clearUserRequest");
      viewModel.clearUserRequest();
    }
    setState(() {
      plEnabled = !plEnabled;
    });
  }

  isDisabled(_ViewModel viewModel) {
    if (plEnabled && viewModel.userRequest != null) {
      return !RegExp(r"^[0-9]{6}").hasMatch(_code);
    } else {
      print(_email);
      return !RegExp(
              r"^[a-zA-Z0-9.a-zA-Z0-9.!#$%&'*+-/=?^_`{|}~]+@[a-zA-Z0-9-.-_]+\.[a-zA-Z]+")
          .hasMatch(_email);
    }
  }

  @override
  Widget build(BuildContext context) {
    return (StoreConnector<T, _ViewModel>(
        converter: (Store<T> store) => _ViewModel.create(store),
        onDidChange: (vm1, vm2) {
          if (vm2.userRequest != null && !preventAutofocus) {
            preventAutofocus = true;
            _codeFocus.requestFocus();
          }
        },
        builder: (BuildContext context, _ViewModel viewModel) {
          final isDark = Theme.of(context).brightness == Brightness.dark;
          var buttonLabel = plEnabled ? "Send Code" : "Signin";
          if (viewModel.userRequest != null) {
            buttonLabel = "Submit Code";
          }
          return Form(
            key: _formKey,
            child: Wrap(
              runSpacing: 10.0,
              children: <Widget>[
                if ((viewModel.userRequest == null) || !plEnabled)
                  TextFormField(
                    autofocus: true,
                    controller: emailController,
                    keyboardAppearance: Theme.of(context).brightness,
                    focusNode: _emailFocus,
                    onChanged: this.handleEmailChange,
                    decoration: const InputDecoration(
                      hintText: 'Enter your email',
                    ),
                    onFieldSubmitted: (value) {
                      FocusScope.of(context).unfocus();
                      FocusScope.of(context).requestFocus(_passwordFocus);
                    },
                    keyboardType: TextInputType.emailAddress,
                    validator: (value) {
                      if (value!.isEmpty) {
                        return 'Please enter some text';
                      }
                      return null;
                    },
                  ),
                if (viewModel.userRequest != null && plEnabled)
                  Center(
                    child: Card(
                        child: Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Wrap(
                        crossAxisAlignment: WrapCrossAlignment.center,
                        spacing: 4,
                        children: [
                          Icon(Icons.email_outlined, size: 16),
                          Text(
                              "You should have received a code by email or SMS",
                              style: Theme.of(context).textTheme.caption),
                        ],
                      ),
                    )),
                  ),
                if (viewModel.userRequest != null && plEnabled)
                  TextFormField(
                    controller: codeController,
                    focusNode: _codeFocus,
                    onChanged: this.handleCodeChange,
                    keyboardAppearance: Theme.of(context).brightness,
                    keyboardType: TextInputType.number,
                    decoration: const InputDecoration(
                      hintText: 'Enter the 6 digit code',
                    ),
                    validator: (value) {
                      if (value != null && value.isEmpty) {
                        return 'Please enter some text';
                      }
                      return null;
                    },
                  ),
                if (!plEnabled)
                  TextFormField(
                    controller: passwordController,
                    focusNode: _passwordFocus,
                    keyboardAppearance: Theme.of(context).brightness,
                    obscureText: true,
                    decoration: const InputDecoration(
                      hintText: 'Enter your password',
                    ),
                    onFieldSubmitted: (value) {
                      _passwordFocus.unfocus();
                    },
                    validator: (value) {
                      if (value != null && value.isEmpty) {
                        return 'Please enter some text';
                      }
                      return null;
                    },
                  ),
                Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    if (viewModel.userRequest != null)
                      FlatButton(
                          onPressed: () {
                            viewModel.clearUserRequest();
                            FocusScope.of(context).unfocus();
                            _emailFocus.requestFocus();
                          },
                          child: Text("Cancel")),
                    ApiButton<T>(
                        label: buttonLabel,
                        disabled: isDisabled(viewModel),
                        color: Theme.of(context).accentColor,
                        apiCall: () {
                          FocusScope.of(context).unfocus();
                          if (_formKey.currentState!.validate()) {
                            if (plEnabled) {
                              if (viewModel.userRequest != null) {
                                // Send code if present with 6 digits
                                return viewModel
                                    .postPLSignin(this.codeController.text);
                              } else {
                                return viewModel.postRequestPLCode(
                                    this.emailController.text);
                              }
                            } else {
                              return viewModel.signin(this.emailController.text,
                                  this.passwordController.text);
                            }
                          }
                          return null;
                        }),
                  ],
                ),
                GestureDetector(
                  onTap: () => togglePlEnabled(viewModel),
                  child: Center(
                    child: Opacity(
                      opacity: 0.58,
                      child: Container(
                        child: Padding(
                          padding: const EdgeInsets.only(
                              left: 4.0, right: 4.0, top: 2, bottom: 2),
                          child: Wrap(
                            children: [
                              Padding(
                                padding: const EdgeInsets.only(right: 4.0),
                                child: Image.asset(
                                  plEnabled
                                      ? 'lib/assets/key.png'
                                      : 'lib/assets/wand.png',
                                  package: "account_kit",
                                  width: 12,
                                  color: isDark ? Colors.white : Colors.white,
                                ),
                              ),
                              Text(
                                plEnabled
                                    ? "Signin with my password"
                                    : "Send me a password free sign in code",
                                style: Theme.of(context).textTheme.caption,
                              ),
                            ],
                          ),
                        ),
                      ),
                    ),
                  ),
                ),
              ],
            ),
          );
        }));
  }
}

class _ViewModel {
  final Function(String, String) signin;
  final Function(String) postRequestPLCode;
  final Function(String) postPLSignin;
  final bool signinLoading = false;
  final Request? userRequest;
  final Function() clearUserRequest;

  _ViewModel(this.signin, this.postRequestPLCode, this.postPLSignin,
      this.userRequest, this.clearUserRequest);

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
        (email, password) => PostSignin(email, password),
        (email) => PostRequestPLCode(email),
        (code) => PostPLSignin(code),
        store.state.accountKitState.userState.request,
        () => store.dispatch(ClearUserRequestAction()));
  }
}

==> 3.0.1.tar.gz: lib/ui/views/signinForm.dart <==
import 'package:account_kit/entities/request/request.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/user/userActions.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:puma_network/widget/apiButton.dart';
import 'package:redux/redux.dart';

class SigninForm<T extends VirtualAccountState> extends StatefulWidget {
  // final void Function(String, String) onSubmit;
  final bool plEnabled;
  SigninForm({this.plEnabled = false});

  @override
  _SigninFormStateWidget createState() => _SigninFormStateWidget<T>();
}

/// This is the private State class that goes with SigninForm.
class _SigninFormStateWidget<T extends VirtualAccountState>
    extends State<SigninForm> {
  final _formKey = GlobalKey<FormState>();
  TextEditingController emailController = TextEditingController();
  TextEditingController codeController = TextEditingController();
  TextEditingController passwordController = TextEditingController();
  final FocusNode _emailFocus = FocusNode();
  final FocusNode _codeFocus = FocusNode();
  final FocusNode _passwordFocus = FocusNode();
  String _email = '';
  String _code = '';
  bool preventAutofocus = false;
  bool plEnabled = false;
  // Todo: Get Env default PLEnabled

  @override
  void initState() {
    super.initState();
    plEnabled = widget.plEnabled;
  }

  handleEmailChange(value) {
    setState(() {
      _email = value;
    });
  }

  handleCodeChange(value) {
    setState(() {
      _code = value;
    });
  }

  togglePlEnabled(_ViewModel viewModel) {
    if (plEnabled) {
      print("Dispatching clearUserRequest");
      viewModel.clearUserRequest();
    }
    setState(() {
      plEnabled = !plEnabled;
    });
  }

  isDisabled(_ViewModel viewModel) {
    if (plEnabled && viewModel.userRequest != null) {
      return !RegExp(r"^[0-9]{6}").hasMatch(_code);
    } else {
      print(_email);
      return !RegExp(
              r"^[a-zA-Z0-9.a-zA-Z0-9.!#$%&'*+-/=?^_`{|}~]+@[a-zA-Z0-9-.-_]+\.[a-zA-Z]+")
          .hasMatch(_email);
    }
  }

  @override
  Widget build(BuildContext context) {
    return (StoreConnector<T, _ViewModel>(
        converter: (Store<T> store) => _ViewModel.create(store),
        onDidChange: (vm1, vm2) {
          if (vm2.userRequest != null && !preventAutofocus) {
            preventAutofocus = true;
            _codeFocus.requestFocus();
          }
        },
        builder: (BuildContext context, _ViewModel viewModel) {
          final isDark = Theme.of(context).brightness == Brightness.dark;
          var buttonLabel = plEnabled ? "Send Code" : "Signin";
          if (viewModel.userRequest != null) {
            buttonLabel = "Submit Code";
          }
          return Form(
            key: _formKey,
            child: Wrap(
              runSpacing: 10.0,
              children: <Widget>[
                if ((viewModel.userRequest == null) || !plEnabled)
                  TextFormField(
                    autofocus: true,
                    controller: emailController,
                    keyboardAppearance: Theme.of(context).brightness,
                    focusNode: _emailFocus,
                    onChanged: this.handleEmailChange,
                    decoration: const InputDecoration(
                      hintText: 'Enter your email',
                    ),
                    onFieldSubmitted: (value) {
                      FocusScope.of(context).unfocus();
                      FocusScope.of(context).requestFocus(_passwordFocus);
                    },
                    keyboardType: TextInputType.emailAddress,
                    validator: (value) {
                      if (value!.isEmpty) {
                        return 'Please enter some text';
                      }
                      return null;
                    },
                  ),
                if (viewModel.userRequest != null && plEnabled)
                  Center(
                    child: Card(
                        child: Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Wrap(
                        crossAxisAlignment: WrapCrossAlignment.center,
                        spacing: 4,
                        children: [
                          Icon(Icons.email_outlined, size: 16),
                          Text(
                              "You should have received a code by email or SMS",
                              style: Theme.of(context).textTheme.caption),
                        ],
                      ),
                    )),
                  ),
                if (viewModel.userRequest != null && plEnabled)
                  TextFormField(
                    controller: codeController,
                    focusNode: _codeFocus,
                    onChanged: this.handleCodeChange,
                    keyboardAppearance: Theme.of(context).brightness,
                    keyboardType: TextInputType.number,
                    decoration: const InputDecoration(
                      hintText: 'Enter the 6 digit code',
                    ),
                    validator: (value) {
                      if (value != null && value.isEmpty) {
                        return 'Please enter some text';
                      }
                      return null;
                    },
                  ),
                if (!plEnabled)
                  TextFormField(
                    controller: passwordController,
                    focusNode: _passwordFocus,
                    keyboardAppearance: Theme.of(context).brightness,
                    obscureText: true,
                    decoration: const InputDecoration(
                      hintText: 'Enter your password',
                    ),
                    onFieldSubmitted: (value) {
                      _passwordFocus.unfocus();
                    },
                    validator: (value) {
                      if (value != null && value.isEmpty) {
                        return 'Please enter some text';
                      }
                      return null;
                    },
                  ),
                Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    if (viewModel.userRequest != null)
                      FlatButton(
                          onPressed: () {
                            viewModel.clearUserRequest();
                            FocusScope.of(context).unfocus();
                            _emailFocus.requestFocus();
                          },
                          child: Text("Cancel")),
                    ApiButton(
                        label: buttonLabel,
                        disabled: isDisabled(viewModel),
                        color: Theme.of(context).accentColor,
                        apiCall: () {
                          FocusScope.of(context).unfocus();
                          if (_formKey.currentState!.validate()) {
                            if (plEnabled) {
                              if (viewModel.userRequest != null) {
                                // Send code if present with 6 digits
                                return viewModel
                                    .postPLSignin(this.codeController.text);
                              } else {
                                return viewModel.postRequestPLCode(
                                    this.emailController.text);
                              }
                            } else {
                              return viewModel.signin(this.emailController.text,
                                  this.passwordController.text);
                            }
                          }
                          return null;
                        }),
                  ],
                ),
                GestureDetector(
                  onTap: () => togglePlEnabled(viewModel),
                  child: Center(
                    child: Opacity(
                      opacity: 0.58,
                      child: Container(
                        child: Padding(
                          padding: const EdgeInsets.only(
                              left: 4.0, right: 4.0, top: 2, bottom: 2),
                          child: Wrap(
                            children: [
                              Padding(
                                padding: const EdgeInsets.only(right: 4.0),
                                child: Image.asset(
                                  plEnabled
                                      ? 'lib/assets/key.png'
                                      : 'lib/assets/wand.png',
                                  package: "account_kit",
                                  width: 12,
                                  color: isDark ? Colors.white : Colors.white,
                                ),
                              ),
                              Text(
                                plEnabled
                                    ? "Signin with my password"
                                    : "Send me a password free sign in code",
                                style: Theme.of(context).textTheme.caption,
                              ),
                            ],
                          ),
                        ),
                      ),
                    ),
                  ),
                ),
              ],
            ),
          );
        }));
  }
}

class _ViewModel {
  final Function(String, String) signin;
  final Function(String) postRequestPLCode;
  final Function(String) postPLSignin;
  final bool signinLoading = false;
  final Request? userRequest;
  final Function() clearUserRequest;

  _ViewModel(this.signin, this.postRequestPLCode, this.postPLSignin,
      this.userRequest, this.clearUserRequest);

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
        (email, password) => PostSignin(email, password),
        (email) => PostRequestPLCode(email),
        (code) => PostPLSignin(code),
        store.state.accountKitState.userState.request,
        () => store.dispatch(ClearUserRequestAction()));
  }
}
